from .download import download_zip, download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_file(xz_url + "DEFAULT.blend",down_path)

bl_info = {
    "name" : "Cinepack V2",
    "author" : "Lewis Martin(Asset Pack Creator),Pavan Bhadaja(Add-on Developer)",
    "description" : "",
    "blender" : (2, 80, 0),
    "version" : (2, 2, 1),
    "location" : "3D View > Sidebar(N-panel)",
    "warning" : "",
    "category" : "Camera"
}

import bpy
from bpy.types import Scene, WindowManager
from bpy.app.handlers import persistent
from . import preferences,properties,operators,ui,handlers,presets
from .translation.translate import GetTranslationDict

class Cinepack:
    __slots__ = ('camera_moves_dict','camera_moves_path','camera_moves_enum','selected_categories','categories','thumbnails',
                'filter_name','all_camera_moves_enum','filtered_camera_moves_enum')

def register():
    Scene.cinepack_data = Cinepack()
    Scene.cinepack_data.camera_moves_dict = {}      #stores dictionary of category and camera moves which belongs to it 
                                                    #{category : [(cam_mov1,cam_mov1_dir_path),(cam_mov1,cam_mov1_dir_path)]}
    Scene.cinepack_data.camera_moves_path = {}      #stores dictionary of camera moves and it's directory path
                                                    #{camera_move_name : camera_move_dir_path}
    Scene.cinepack_data.camera_moves_enum = []      #for caching camera_moves,
                                                    #same as scene.Cinepack.camera_moves enum
    Scene.cinepack_data.selected_categories = {}    #to check if camera_moves cache need recalculation, 
                                                    #same as value of scene.Cinepack.categories
    Scene.cinepack_data.categories = []             #cache of categories so we don't need to recalculate it.
                                                    #same as enum of scene.Cinepack.categories
    import bpy.utils.previews
    Scene.cinepack_data.thumbnails = bpy.utils.previews.new() 

    Scene.cinepack_data.filter_name = ''

    Scene.cinepack_data.filtered_camera_moves_enum = []
    from collections import defaultdict
    Scene.cinepack_data.all_camera_moves_enum = defaultdict(lambda:[])

    WindowManager.preset_category = bpy.props.StringProperty(name='Preset Category',description='Name for a category of preset.\nDefault is Cinepack.',default='Cinepack')
    preferences.register()
    properties.register()
    operators.register()
    presets.register()
    ui.register()
    handlers.register()
    try:
        bpy.app.translations.register(__package__, GetTranslationDict())
    except:
        pass

def unregister():
    handlers.unregister()
    ui.unregister()
    presets.unregister()
    operators.unregister()
    properties.unregister()
    bpy.utils.previews.remove(Scene.cinepack_data.thumbnails)
    preferences.unregister()
    # del Scene.cinepack_data
    try:
        bpy.app.translations.unregister(__package__)
    except:
        pass
    