import typing
import bpy,os
from bpy.props import BoolProperty, StringProperty,EnumProperty, IntProperty
from bpy.types import Context, Event, Operator
from .camera_defination import camera_def

class CinePack_OT_append(Operator):
    '''Add Camera move to your scene.'''
    bl_idname = "cinepack.append"
    bl_label = "CinePack Append"
    bl_description = "Add Camera move to your scene."
    bl_options = {"REGISTER", "UNDO"}

    cam_move : StringProperty(name="cam_move",options={'HIDDEN','SKIP_SAVE'})
    def get_blend_file(self,directory):
        for filename in os.listdir(directory):
            if filename.endswith(".blend"):
                return os.path.join(directory,filename)
        else:
            print("No Blend File in Directory : ", directory)
            return None


    def get_proper_filepath(self,name,path):
        def replace_name(name,path,replacement):
            filename = name.replace(' ',replacement)
            filename = filename+".blend"
            filepath = os.path.join(path,filename)
            return (filepath,os.path.exists(filepath))
        
        replacements = [' ','_','']
        filepath_flag = [replace_name(name,path,replacement) for replacement in replacements]
        return next(filter(lambda ff: ff[1],filepath_flag))[0]

    def append_camera_collection(self,context,path,cam_name):
        link = False
        filepath = path
        print(filepath)
        #import saved scene collection
        with bpy.data.libraries.load(filepath, link=link) as (data_from, data_to):
            data_to.collections = [c for c in data_from.collections if c  == 'Camera']
        
        for coll in data_to.collections:
            if coll is not None:
                name = f'CAMERA - {cam_name}'
                coll.name = name
                camera = next(obj for obj in coll.objects if obj.type == 'CAMERA')
                camera.name = name #object name
                camera.data.name = name #camera datablock name
                context.scene.collection.children.link(coll)
                context.scene.camera = camera
                camera.data.is_cinepack = True
                if cam_name in camera_def:
                    camera.data.cinepack_expose_targe = camera_def[cam_name]['expose_target']
                    camera.data.cinepack_use_range = camera_def[cam_name]['use_range']
    
    @classmethod
    def poll(cls, context):
        return context.scene.Cinepack.camera_moves
        # return True

    def execute(self, context):
        if not self.cam_move:
            cam_move = context.scene.Cinepack.camera_moves
            # print("getting cinepack camera_moves enum value", cam_move)
        else:
            cam_move = self.cam_move
        # print(context.scene.cinepack.camera_moves_dict)
        path = context.scene.cinepack_data.camera_moves_path[cam_move]
        # print(f"path to {cam_move} : {path}")
        blend_file_path = self.get_blend_file(path)
        if not blend_file_path:
            self.report({'ERROR'},f"Can not find blend file in directory : {path}")
            return {'CANCELLED'}
        self.append_camera_collection(context,blend_file_path,cam_move)
        return {'FINISHED'}
    
class CinePack_OT_open_gif(Operator):
    '''Open Preview GIF in any image viewer application'''
    bl_idname = "cinepack.open_gif"
    bl_label = "Open in ImageViewer"
    bl_description = "Open Preview GIF in any image viewer application"
    bl_options = {'REGISTER'}

    cam_move : StringProperty(name="cam_move",options={'HIDDEN','SKIP_SAVE'})
    #https://stackoverflow.com/a/35305473/10358314
    def openImage(self,context,path):
        import sys
        import subprocess

        imageViewerFromCommandLine = context.preferences.addons[__package__].preferences.ext_img_editor
        if(not imageViewerFromCommandLine):
            imageViewerFromCommandLine = {'linux':['xdg-open'],
                                    'win32':['explorer'],
                                    'darwin':['qlmanage', '-p']}[sys.platform]
        subprocess.run([*imageViewerFromCommandLine,path])

    @classmethod
    def poll(cls, context):
        return context.scene.Cinepack.camera_moves
        # return True

    def execute(self, context):
        if not self.cam_move:
            cam_move = context.scene.Cinepack.camera_moves
        else:
            cam_move = self.cam_move
        path = context.scene.cinepack_data.camera_moves_path[cam_move]
        proper_filepath = os.path.join(path,'Preview.MP4')
        if not(os.path.isfile(proper_filepath)):
            self.report({'ERROR'},"Preview Not Found!!!")
            return {'CANCELLED'}
        self.openImage(context,proper_filepath)
        return {'FINISHED'}

class Cinepack_OT_MakeCameraActiveOperator(Operator):
    '''Make Camera Active'''
    bl_idname = "cinepack.makecameraactive"
    bl_label = "Make Camera Active"
    bl_description = "Make Camera Active"
    bl_options = {'REGISTER','UNDO'}
    @classmethod
    def poll(self,context):
        scn = context.scene
        current_camera = scn.cinepack_cameras
        return current_camera != 'NO CINEPACK CAMERA'

    def execute(self, context):
        scn = context.scene
        current_camera = scn.cinepack_cameras
        camera_obj = next(obj for obj in bpy.data.objects if obj.type == 'CAMERA' and obj.data.name == current_camera)
        scn.camera = camera_obj
        return {'FINISHED'}

class Cinepack_OT_apply_anim_modification(Operator):
    '''Change Animation based on given input'''
    bl_idname = "cinepack.apply_anim_modification"
    bl_label = "Apply Animation"
    bl_description = "Change Animation based on given input"
    bl_options = {'REGISTER','UNDO'}

    @classmethod
    def poll(self,context):
        name = context.scene.cinepack_cameras
        return name in bpy.data.collections and name in bpy.data.collections[name].objects

    def execute(self, context):
        name = context.scene.cinepack_cameras
        coll = bpy.data.collections[name]
        cp_mods = context.scene.cinepack_mods
        for obj in coll.objects:
            # print(obj.name)
            if obj.animation_data.action:
                action = obj.animation_data.action
                fc_list = [fc for fc in action.fcurves]
                # print(fc_list)
                if obj.data and obj.data.animation_data:
                    action = obj.data.animation_data.action
                if action:
                    fc_list.extend([fc for fc in action.fcurves])
                for fc in fc_list:
                    for i in range(len(fc.keyframe_points)):
                        fc.keyframe_points[i].co_ui[0] += cp_mods.offset
            
        return {'FINISHED'}

class CinePack_OT_save_move(Operator):
    '''Save Camera Move for later use.'''
    bl_idname = "cinepack.save_move"
    bl_label = "Save Move"
    bl_description = "Save Camera Move for later use."
    bl_options = {"REGISTER"}

    def category_items(self,context):
        enum_items = []
        
        directory = context.preferences.addons[__package__].preferences.filepath
        if not directory:
            return enum_items
        #Check for cached categories, return it if available
        if (len(context.scene.cinepack_data.categories) > 0):
            return context.scene.cinepack_data.categories

        print(f"Scanning directory: {directory}")
        categories = [ (f.name,f.path) for f in os.scandir(directory) if f.is_dir() ]
        for i,cat in enumerate(categories):
            cat_name,cat_path = cat
            enum_items.append((cat_name,cat_name,'',2**(i+1)))
        return enum_items

    category_enum : EnumProperty(items = category_items, name="Category", description="Category of new camera move")
    move_name : StringProperty(name = 'Move Name',description='Name which is used to describe Camera Move',)
    overwrite: BoolProperty(name='Overwrite',description='If move already exist then overwrite it', default=False, options={'SKIP_SAVE'})

    @classmethod
    def poll(self,context):
        return context.preferences.addons[__package__].preferences.filepath

    def execute(self, context):
        destination_folder = context.preferences.addons[__package__].preferences.filepath
        # destination_folder = r"D:\Softwares\Blender\Addon\Cinepack\CinePack_assets"
        category = self.category_enum
        name = self.move_name
        if not category:
            self.report({'ERROR'},"Please select Category!!!")
            return {'CANCELLED'}
        if not name:
            self.report({'ERROR'},"Empty Name is Not Allowed!!!")
            return {'CANCELLED'}
        filename = name+'.blend'
        dir_path = os.path.join(destination_folder,category,name)
        file_path = os.path.join(dir_path,filename)
        try:
            os.makedirs(dir_path,exist_ok = self.overwrite)
        except FileExistsError as e:
            self.report({'ERROR'},"Directory with given name already exist!!!")
            print('-'*69)
            print(e)
            print('-'*69)
            import traceback
            traceback.print_exc()
            print('-'*69)
            return {'CANCELLED'}
        except NotADirectoryError as e:
            self.report({'ERROR'},"Given Name is not Valid for creating directory")
            print('-'*69)
            print(e)
            print('-'*69)
            import traceback
            traceback.print_exc()
            print('-'*69)
            return {'CANCELLED'}

        print(file_path)
        bpy.data.libraries.write(file_path, set(context.selected_objects), path_remap='RELATIVE', fake_user=True)

        link_to_scene = os.path.join(os.path.dirname(os.path.abspath(__file__)),'link_to_scene.py')
        from subprocess import run
        run(['blender','-b' , file_path, '--python', link_to_scene])
        context.scene.Cinepack.force_update = True
        context.scene.Cinepack.get_cp_categories(context)
        return {'FINISHED'}
    
    def invoke(self, context, event):
        include_camera = any(obj.type == 'CAMERA' for obj in context.selected_objects)
        if not include_camera:
            self.report({"ERROR"},"Atleast 1 Camera is required")
            return {'CANCELLED'}
        if context.scene.Cinepack.categories:
            self.category_enum = next(iter(context.scene.Cinepack.categories))

        return context.window_manager.invoke_props_dialog(self)


class CinePack_OT_draw_camera(bpy.types.Operator):
    bl_idname = "cinepack.draw_camera"
    bl_label = "Draw"
    bl_description = "Draw camera path\nUse Error threshold property to smooth out the drawn curve. You can find it in redo panel after drawing the curve"
    bl_options = {"REGISTER", "UNDO"}

    start_frame: IntProperty(min=0, default= 1)
    end_frame: IntProperty(min=0, default= 120)
    linear_animation: BoolProperty(default=True, description='Decides if interpolation is Linear or Bezier')
    use_scene_camera: BoolProperty(default=False, description='If this is checked then scene\'s active camera is used other wise new camera is added')
    
    @classmethod
    def poll(cls, context):
        return context.area.type == 'VIEW_3D' and context.object.mode == 'OBJECT'

    def execute(self, context):
        look_at = bpy.data.objects.new('Look At',None)
        context.scene.collection.objects.link(look_at)
        
        curve_data = bpy.data.curves.new('Camera_Path', 'CURVE')
        curve_obj = bpy.data.objects.new('Camera_Path',curve_data)
        context.scene.collection.objects.link(curve_obj)
        
        #Step 2
        camera = context.scene.camera
        if camera is None or self.use_scene_camera is False:
            camera_data = bpy.data.cameras.new('Camera')
            camera = bpy.data.objects.new('Camera',camera_data)
            camera.data.is_cinepack = True
            camera.data.cinepack_expose_targe = True
            context.scene.camera = camera
            context.scene.collection.objects.link(camera)
        
        follow_track = camera.constraints.new('FOLLOW_PATH')
        follow_track.target = curve_obj
        follow_track.use_fixed_location = True
        follow_track.use_curve_follow = True
        
        track_to = camera.constraints.new('TRACK_TO')
        track_to.target = look_at

        frames = [self.start_frame,self.end_frame]
        start_frame,end_frame = min(frames),max(frames)

        follow_track.offset_factor = 0.0
        follow_track.keyframe_insert('offset_factor',frame = start_frame)
        follow_track.offset_factor = 1.0
        follow_track.keyframe_insert('offset_factor',frame = end_frame)
        for fcurve in camera.animation_data.action.fcurves:
            start_kp,end_kp = fcurve.keyframe_points[0],fcurve.keyframe_points[-1]
            start_kp.interpolation = 'LINEAR' if self.linear_animation else 'BEZIER'
            end_kp.interpolation = 'LINEAR' if self.linear_animation else 'BEZIER'

        # Step 3
        bpy.ops.object.select_all(action='DESELECT')
        context.view_layer.objects.active = curve_obj
        curve_obj.select_set(True)
        bpy.ops.object.mode_set(mode='EDIT')
        override_context = context.copy()
        area = [area for area in context.screen.areas if area.type == "VIEW_3D"][0]
        override_context['area'] = area
        override_context['region'] = area.regions[-1]
        bpy.ops.curve.draw(override_context,'INVOKE_DEFAULT')
        
        return {'FINISHED'}


    def invoke(self, context: Context, event: Event):
        return context.window_manager.invoke_props_dialog(self)

classes = [CinePack_OT_append, CinePack_OT_open_gif, Cinepack_OT_MakeCameraActiveOperator, Cinepack_OT_apply_anim_modification,
            CinePack_OT_save_move, CinePack_OT_draw_camera]
def register():
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
