import bpy
from bpy.types import Operator, Menu
from bl_operators.presets import AddPresetBase
from pathlib import Path

class CINEPACK_MT_render_presets(Menu):
    bl_label = "Render Presets"
    preset_subdir = "render/cinepack"
    preset_operator = "script.execute_preset"
    preset_add_operator = "cinepack.render_preset_add"
    draw = Menu.draw_preset

class CinePack_OT_render_Preset(AddPresetBase, Operator):
    '''Add scene resolution and aspect ratio Preset'''
    bl_idname = "cinepack.render_preset_add"
    bl_label = "Add Scene Resolution Preset"
    preset_menu = "CINEPACK_MT_render_presets"

    # variable used for all preset values
    preset_defines = [
        "scene = bpy.context.scene"
    ]

    # properties to store in the preset
    preset_values = [
        "scene.render.resolution_x",
        "scene.render.resolution_y",
        "scene.render.resolution_percentage",
        "scene.render.pixel_aspect_x",
        "scene.render.pixel_aspect_y",
    ]

    # where to store the preset
    preset_subdir = "render/cinepack"

classes = (
    CINEPACK_MT_render_presets,
    CinePack_OT_render_Preset,
)


def register():
    for cls in classes:
        bpy.utils.register_class(cls)

    target_path = Path("presets")/'render'/'cinepack'
    target_path = bpy.utils.user_resource('SCRIPTS', path=str(target_path))
    target_path = Path(target_path)

    if target_path.exists() and target_path.is_dir():
        return

    target_path = Path("presets")/'render'/'cinepack'
    target_path = bpy.utils.user_resource('SCRIPTS', path=str(target_path), create=True)
    target_path = Path(target_path)
    import shutil
    presets = Path(__file__).parent / 'presets'
    print(presets, target_path)
    print(f"CINEPACK: copying preset files from {presets} to {target_path}")
    for file in presets.iterdir():
        shutil.copy2(file, target_path)
    print("CINEPACK: copied the presets files")

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
