import bpy,os
from bpy.types import Camera, PropertyGroup, Scene
from bpy.props import BoolProperty, CollectionProperty, EnumProperty, FloatProperty, IntProperty, PointerProperty, StringProperty

class CinepackSearch(PropertyGroup):
    name : StringProperty(name="filter",)
    category: StringProperty(name="category",)

class CinepackProperties(PropertyGroup):
    def get_image_file(self,directory):
        for filename in os.listdir(directory):
            if filename.endswith(".png") or \
                filename.endswith(".PNG") or \
                filename.endswith(".jpeg") or \
                filename.endswith(".jpg"):
                return os.path.join(directory,filename)
        else:
            # print("No Image found in directory : ", directory)
            return None

    def get_cp_categories(self,context):
        enum_items = []
        # print(self.force_update)
        directory = context.preferences.addons[__package__].preferences.filepath
        if not directory:
            return enum_items
        #Check for cached categories, return it if available
        if (len(context.scene.cinepack_data.categories) > 0 and not self.force_update):
            return context.scene.cinepack_data.categories

        self.force_update = False
        print(f"Scanning directory: {directory}")
        categories = [ (f.name,f.path) for f in os.scandir(directory) if f.is_dir() ]
        for i,cat in enumerate(categories):
            cat_name,cat_path = cat
            enum_items.append((cat_name,cat_name,'',2**(i+1)))
            moves = [ (f.name,f.path) for f in os.scandir(cat_path) if f.is_dir() ]
            Scene.cinepack_data.camera_moves_dict[cat_name] = moves

        #Caching categories
        Scene.cinepack_data.categories = enum_items
        path = os.path.join(os.path.dirname(os.path.abspath(__file__)),'default.jpg')

        #Thumbnail generation
        # import bpy.utils.previews
        pcoll = Scene.cinepack_data.thumbnails
        if not pcoll.get('DEFAULT'): 
            pcoll.load('DEFAULT',path,'IMAGE')
        for cat_name,cat_path in categories:
            for move in Scene.cinepack_data.camera_moves_dict[cat_name]:
                path = self.get_image_file(move[1])
                if not path:
                    continue
                if not pcoll.get(f'{cat_name}_{move[0]}'):
                    pcoll.load(f'{cat_name}_{move[0]}',path,'IMAGE')
        Scene.cinepack_data.thumbnails = pcoll
        
        #cache all camera move enum for grid-layout search
        for cat_name,cat_path in categories:
            for i,move in enumerate(Scene.cinepack_data.camera_moves_dict[cat_name]):
                thumb = pcoll.get(f'{cat}_{move[0]}')
                icon = pcoll['DEFAULT'].icon_id
                if thumb:
                    icon = pcoll[f'{cat}_{move[0]}'].icon_id
                Scene.cinepack_data.all_camera_moves_enum[cat_name].append((move[0],move[0],'',icon,i))
                Scene.cinepack_data.camera_moves_path[move[0]] = move[1]

        #force update deps graph update so search collection(camera_move_collection) gets filled
        # bpy.ops.wm.redraw_timer(type='DRAW',iterations=1)
        # dg = context.evaluated_depsgraph_get()
        # dg.update()
        # context.view_layer.update()
        self.get_camera_moves(context)
        bpy.context.view_layer.update() 
        return Scene.cinepack_data.categories

    def update_cp_categories(self,context):
        # print("updating category")
        scn = context.scene
        cinepack_data = scn.cinepack_data
        self.get_camera_moves(context)
        cinepack_data.selected_categories = scn.Cinepack.categories
        
    def get_camera_moves(self,context):
        enum_items = []

        directory = context.preferences.addons[__package__].preferences.filepath
        scn = context.scene
        cinepack_data = scn.cinepack_data
        categories = scn.Cinepack.categories

        if(categories == cinepack_data.selected_categories): 
            #returning cached enum
            return cinepack_data.camera_moves_enum

        # for cat in categories:
            # enum_items.extend(context.scene.cinepack_data.all_camera_moves_enum[cat])

        thumbnails = context.scene.cinepack_data.thumbnails
        for cat in categories:
            for i,move in enumerate(cinepack_data.camera_moves_dict[cat]):
                thumb = thumbnails.get(f'{cat}_{move[0]}')
                icon = thumbnails['DEFAULT'].icon_id #'OUTLINER_DATA_CAMERA'
                if thumb:
                    icon = thumbnails[f'{cat}_{move[0]}'].icon_id
                enum_items.append((move[0],move[0],'',icon,i))
                cinepack_data.camera_moves_path[move[0]] = move[1]

        #caching enums for future
        cinepack_data.camera_moves_enum = enum_items
        cinepack_data.filtered_camera_moves_enum = enum_items
        return enum_items

    def update_filter_string(self,context):
        # print("updating name-filter...")
        cp_props = context.scene.Cinepack
        cp_data = context.scene.cinepack_data
        if not cp_props.filter_string:
            filter_categories = cp_props.categories
            l = []
            for cat in filter_categories:
                l.extend(cp_data.all_camera_moves_enum[cat])
            cp_data.filtered_camera_moves_enum = l
            context.area.tag_redraw()
            return 
        # print(cp_props.categories)
        filter_category = cp_props.camera_move_collection[cp_props.filter_string].category
        # print(filter_category,cp_props.filter_string)
        if not filter_category in cp_props.categories:
            if cp_props.categories:
                # s = set(cp_props.categories)
                # s = {filter_category} | s
                # cp_props.categories = s
                cp_props.categories = {filter_category}
            else:
                cp_props.categories = {filter_category}
            
        
        # print(cp_props.categories)
        # print(cp_props.camera_moves)
        cp_props.camera_moves = cp_props.filter_string
        cp_data.filtered_camera_moves_enum = list(filter(lambda move :cp_props.filter_string == move[1], cp_data.all_camera_moves_enum[filter_category]))
        # print(cp_props.camera_moves)
    

    force_update : BoolProperty(name='Force Update',description='Force Update Category every draw',default=False)
    categories : EnumProperty(items = get_cp_categories, name="Categories", description="Categories", options={'ENUM_FLAG'},update=update_cp_categories)
    camera_moves : EnumProperty(items = get_camera_moves,name="Camera Move", description="Camera Move to append",default=0)
    filter_string : StringProperty(name="Filter String",description="Search item", update=update_filter_string)
    camera_move_collection : CollectionProperty(type=CinepackSearch)
    scale_grid_layout : FloatProperty(name = "Icon Scale",description="Display Scale of the Thumbnails",
                                    min=1.0,max=100.0,soft_min=1.0,soft_max=10.0,default=5.0)
    auto_col_grid_layout : BoolProperty(name="auto",default=True,description="Set Column numbers to Auto")
    columns_grid_layout : IntProperty(name="Columns",description="Display Columns",default=3,min=1,max=6)


class CinepackModifier(PropertyGroup):
    start_frame : IntProperty(name="Start frame")
    end_frame : IntProperty(name="End frame")
    offset : IntProperty(name="Offset frame")
    use_custom_range : BoolProperty(name="use custom frame range",default=False)

classes = [CinepackSearch, CinepackProperties, CinepackModifier]

def get_cinepack_camera(self,context):
    items = [(cam.name,cam.name,'') for cam in bpy.data.objects if cam.type == 'CAMERA' and cam.data.is_cinepack and cam.data.users]
    if not items:
        # print("No items, appending dummy")
        items.append(('NO CINEPACK CAMERA','--NO CINEPACK CAMERA--',''))
    # print('ITEMS : ', items)
    return items

def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    Scene.Cinepack = PointerProperty(type=CinepackProperties)
    Scene.cinepack_mods = PointerProperty(type=CinepackModifier)
    Camera.is_cinepack = bpy.props.BoolProperty(name="is_cinepack",default=False)
    Camera.cinepack_expose_targe = bpy.props.BoolProperty(name="cinepack_expose_targe",default=False)
    Camera.cinepack_use_range = bpy.props.BoolProperty(name="cinepack_use_range",default=False)
    Scene.cinepack_cameras = bpy.props.EnumProperty(items = get_cinepack_camera)

def unregister():
    del Scene.cinepack_cameras
    del Camera.cinepack_expose_targe
    del Camera.is_cinepack
    del Scene.cinepack_mods
    del Scene.Cinepack
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
    
