from collections import defaultdict
from posixpath import split
import bpy
from bpy.types import Context, Panel
from .operators import CinePack_OT_append, CinePack_OT_open_gif, CinePack_OT_save_move
from .presets import CinePack_OT_render_Preset
from bl_ui.utils import PresetPanel
from pathlib import Path


class CINEPACK_PT_GL_Settings(Panel):
    bl_idname = "CINEPACK_PT_GL_Settings"
    bl_label = "Settings"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "CinePack"
    bl_options = {"INSTANCED"}

    def draw(self, context):
        layout = self.layout
        cp_props = context.scene.Cinepack
        col = layout.column()
        col.prop(cp_props, "scale_grid_layout", text="Scale")
        col.prop(cp_props, "auto_col_grid_layout", text="Use Auto")
        row = col.row()
        row.active = not cp_props.auto_col_grid_layout
        row.prop(cp_props, "columns_grid_layout", text="Columns")


def draw_visual(self, context, cp_props, cp_data):
    layout = self.layout
    row = layout.row()
    row.prop_search(
        cp_props,
        "filter_string",
        cp_props,
        "camera_move_collection",
        icon="VIEWZOOM",
        text="",
    )
    row.popover("CINEPACK_PT_GL_Settings", text="", icon="SETTINGS")

    thresholds = (8, 10, 14, 30)
    values = (1, 2, 4, 8)
    col_num = self.get_panel_width("UI", thresholds, values)
    col = layout.grid_flow(columns=col_num[0], align=True)
    col.prop(cp_props, "categories", icon="CAMERA_DATA", expand=True)

    if cp_props.categories:
        box = layout.box()
        grid = box.grid_flow(
            row_major=True,
            columns=(
                0 if cp_props.auto_col_grid_layout else cp_props.columns_grid_layout
            ),
            even_columns=True,
            even_rows=False,
            align=False,
        )
        for (
            identifier,
            name,
            description,
            icon,
            id,
        ) in cp_data.filtered_camera_moves_enum:
            col = grid.column(align=True)

            sub_box = col.box()
            split = sub_box.split(factor=0.75)
            row = split.row()
            row.label(text=name)

            row = split.row(align=True)
            row.alignment = "RIGHT"
            op = row.operator(
                CinePack_OT_open_gif.bl_idname,
                text="",
                icon="PLAY",
            )
            op.cam_move = identifier
            op = row.operator(CinePack_OT_append.bl_idname, text="", icon="IMPORT")
            op.cam_move = identifier

            sub_box = col.box()
            try:
                sub_box.template_icon(icon, scale=cp_props.scale_grid_layout)
            except:
                icon = context.scene.cinepack_data.thumbnails["DEFAULT"].icon_id
                # cam_icon = bpy.types.UILayout.bl_rna.functions["prop"].parameters["icon"].enum_items.get('OUTLINER_DATA_CAMERA')
                # icon = cam_icon.value
                sub_box.template_icon(icon, scale=cp_props.scale_grid_layout)
            col.separator()
            # sub_box = col.box()
            # col = sub_box.column(align=True)
            # row = sub_box.row()
            # row.alignment = 'CENTER'
            # op = row.operator(CinePack_OT_open_gif.bl_idname,text="Preview",icon="PLAY")
            # op.cam_move = identifier
            # op = row.operator(CinePack_OT_append.bl_idname,text="Import",icon="IMPORT")
            # op.cam_move = identifier


def draw_compact(self, context, cp_props, cp_data):
    layout = self.layout
    row = layout.row()
    row.prop_search(
        cp_props,
        "filter_string",
        cp_props,
        "camera_move_collection",
        icon="VIEWZOOM",
        text="",
    )

    thresholds = (8, 10, 14, 30)
    values = (1, 2, 4, 8)
    col_num = self.get_panel_width("UI", thresholds, values)
    col = layout.grid_flow(columns=col_num[0], align=True)
    col.prop(cp_props, "categories", icon="CAMERA_DATA", expand=True)
    # Compact UI
    if cp_props.categories:
        box = layout.box()
        scale_num = self.get_panel_width("UI", (4, 8, 10, 14, 16), (6, 7, 8, 10, 10))
        box.template_icon_view(
            cp_props, "camera_moves", show_labels=True, scale=scale_num[0]
        )

        row = box.row()
        row.alignment = "CENTER"
        row.label(text=cp_props.camera_moves, icon="OUTLINER_DATA_CAMERA")

        fac = self.check_width("UI", 14, 1)
        row = layout.row()
        row.scale_y = 2.0
        row.operator(CinePack_OT_open_gif.bl_idname, text="Preview", icon="PLAY")
        if not fac:
            row = layout.row()
        row.scale_y = 2.0
        row.alert = True
        row.operator(CinePack_OT_append.bl_idname, text="Import", icon="IMPORT")


class CINEPACK_PT_render_presets(PresetPanel, Panel):
    bl_label = "RenderPresets"
    preset_subdir = "render/cinepack"
    preset_operator = "script.execute_preset"
    preset_add_operator = "cinepack.render_preset_add"

    @classmethod
    def draw_menu(cls, layout, text=None):
        if text is None:
            text = cls.bl_label

        layout.popover(
            panel=cls.__name__,
            icon="PRESET",
            text=text,
        )

    def custom_draw_preset(self, context):
        target_path = Path("presets") / self.preset_subdir
        target_path = bpy.utils.user_resource(
            "SCRIPTS", path=str(target_path), create=True
        )
        target_path = Path(target_path)

        layout = self.layout
        col = layout.column(align=True)
        col.emboss = "PULLDOWN_MENU"
        col.operator_context = "EXEC_DEFAULT"

        category_preset_map = defaultdict(list)
        for file in target_path.iterdir():
            if file.is_dir():
                continue
            row = col.split(factor=0.9, align=True)
            filename: str = bpy.path.display_name(file.name)
            name_list = filename.split(" ", 1)
            category_preset_map[name_list[0]].append(
                [name_list[-1], str(file), filename]
            )

        for cat, presets in category_preset_map.items():
            col = col.column()
            col.separator()
            col.label(text=cat)
            for displayname, filepath, filename in presets:
                row = col.row()
                op = row.operator(self.preset_operator, text=displayname)
                op.filepath, op.menu_idname = filepath, self.bl_idname
                op = row.operator(self.preset_add_operator, text="", icon="REMOVE")
                op.name = filename
                op.remove_name = True
        layout.separator()
        row = layout.split(factor=0.9)
        col = row.column()
        col.prop(
            context.window_manager, "preset_name", text="", expand=True, slider=True
        )
        col.prop(
            context.window_manager, "preset_category", text="", expand=True, slider=True
        )
        col = row.column()
        col.emboss = "PULLDOWN_MENU"
        col.operator_context = "EXEC_DEFAULT"
        col.scale_y = 2
        op = col.operator(
            self.preset_add_operator,
            text="",
            icon="ADD",
        )

        category = context.window_manager.preset_category
        if category == "" or category.lower() == "cinepack":
            category = "Cinepack"
        op.name = f"{context.window_manager.preset_category}_{context.window_manager.preset_name}"

    def draw(self, context):
        layout = self.layout
        # layout.emboss = 'PULLDOWN_MENU'
        # layout.operator_context = 'EXEC_DEFAULT'
        # bpy.types.Menu.draw_preset(self, context)
        self.custom_draw_preset(context)


class CINEPACK_PT_Main(Panel):
    bl_idname = "CINEPACK_PT_Main"
    bl_label = "CinePack Assets"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "CinePack"

    @classmethod
    def check_width(context, region, threshold, type, max=None):
        area = bpy.context.area
        resolution = bpy.context.preferences.system.ui_scale

        for reg in area.regions:
            if reg.type == region:
                region_width_raw = reg.width

        if not max:
            max = 440

        region_width = region_width_raw - 40
        region_width_int = round(region_width / (20 * resolution))
        region_width_factor = round((region_width_int / max), 3)

        width_type = [region_width, region_width_int, region_width_factor]

        if width_type[type] >= threshold:
            return True
        else:
            return False

    @classmethod
    def get_panel_width(context, region, thresholds, values):
        area = bpy.context.area
        resolution = bpy.context.preferences.system.ui_scale

        for reg in area.regions:
            if reg.type == region:
                region_width_raw = reg.width

        region_width = region_width_raw - 40
        region_width_int = round(region_width / (20 * resolution))

        count = [0, 1]
        count[0] = 1
        count[1] = region_width_int

        for i, val in enumerate(thresholds):
            if region_width_int >= thresholds[i]:
                count[0] = values[i]

        return count

    def draw(self, context):
        layout = self.layout
        cp_props = context.scene.Cinepack
        cp_data = context.scene.cinepack_data
        prefs = context.preferences.addons[__package__].preferences
        if not prefs.filepath:
            col = layout.column()
            col.label(text="Select Cinepack Folder in Addon preference")
        else:
            if prefs.ui_layout == "VISUAL":
                draw_visual(self, context, cp_props, cp_data)
            else:
                draw_compact(self, context, cp_props, cp_data)

            layout.separator()
        box = layout.box()
        box.operator(CinePack_OT_save_move.bl_idname, text="Create Camera Move")


class CINEPACK_PT_Move_Modification(Panel):
    bl_idname = "CINEPACK_PT_Move_Modification"
    bl_label = "CinePack Camera"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "CinePack"

    def draw(self, context):
        layout = self.layout
        row = layout.row()
        row.prop(context.scene, "cinepack_cameras", text="")
        row = layout.row()
        row.operator("cinepack.makecameraactive")
        if (
            context.scene.cinepack_cameras
            and context.scene.cinepack_cameras != "NO CINEPACK CAMERA"
        ):
            box = layout.box()
            row = box.row()
            row.use_property_split = True
            name = context.scene.cinepack_cameras
            camera_obj = bpy.data.objects[name]
            camera = camera_obj.data
            row.prop(
                camera,
                "lens",
            )

            row = box.row()
            row.use_property_split = True
            row.use_property_decorate = False
            row.prop(camera.dof, "use_dof")

            row = box.row()
            row.use_property_split = True
            if not camera.dof.use_dof:
                row.active = False
            row.prop(camera.dof, "focus_object")

        if (
            context.scene.cinepack_cameras
            and context.scene.cinepack_cameras != "NO CINEPACK CAMERA"
        ):
            box = layout.box()
            row = box.row()
            row.use_property_split = True
            name = context.scene.cinepack_cameras
            camera_obj = bpy.data.objects[name]
            camera_data = camera_obj.data
            if camera_data.cinepack_expose_targe:
                row.prop(camera_obj.constraints["Track To"], "target")
            else:
                row = box.row()
                row.alignment = "CENTER"
                row.label(
                    text="Feature Not available for this camera move", icon="MODIFIER"
                )

        layout.operator("cinepack.draw_camera")
        CINEPACK_PT_render_presets.draw_menu(layout, text="Render Presets")

        # layout.row().menu(CINEPACK_MT_render_presets.__name__, text=CINEPACK_MT_render_presets.bl_label)

        # if(context.scene.cinepack_cameras != 'NO CINEPACK CAMERA'):
        #     cp_mods = context.scene.cinepack_mods
        #     box = layout.box()
        #     name = context.scene.cinepack_cameras
        #     camera_data = bpy.data.cameras[name]
        #     if not (name in bpy.data.collections and name in bpy.data.collections[name].objects):
        #         row = box.row()
        #         row.alignment = 'CENTER'
        #         row.alert = True
        #         row.label(text="Camera Object and Camera Collection name must be Same",icon='ERROR')
        #     else:
        #         if(camera_data.cinepack_use_range):
        #             row = box.row()
        #             row.active = not cp_mods.use_custom_range
        #             # row.use_property_split = True
        #             row.prop(cp_mods,"offset")
        #             row = box.row()
        #             row.operator(Cinepack_OT_apply_anim_modification.bl_idname)
        #         else:
        #             row = box.row()
        #             row.alignment = 'CENTER'
        #             row.label(text="Feature Not available for this camera move",icon='MODIFIER')


class CINEPACK_PT_Info(Panel):
    bl_idname = "CINEPACK_PT_Info"
    bl_label = "Info"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "CinePack"
    bl_order = 120

    urls = {
        "documentation": "https://blendermarket.com/products/cinepack-pre-animated-camera-moves/docs",
        "discord": "https://discord.com/invite/B3Ux4sxAzT",
        "community": "https://blenderartists.org/t/addon-40-pre-animated-cinematic-camera-moves/1280362",
    }

    def draw(self, context):
        layout = self.layout
        layout.operator("wm.url_open", text="Documentation", icon="URL").url = (
            self.urls["documentation"]
        )
        layout.operator("wm.url_open", text="Community", icon="URL").url = self.urls[
            "community"
        ]
        layout.operator("wm.url_open", text="Discord", icon="URL").url = self.urls[
            "discord"
        ]


classes = [
    CINEPACK_PT_GL_Settings,
    CINEPACK_PT_Main,
    CINEPACK_PT_Move_Modification,
    CINEPACK_PT_render_presets,
    # CINEPACK_PT_Info,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
